@@ -21,6 +21,7 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include "window.h"
 #include "undo.h"
@@ -193,10 +194,13 @@ gedit_document_set_readonly (GeditDocument *doc, gint readonly)
 {
 	GeditView * nth_view;
 	gint n;
-
+	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
+	g_return_if_fail(doc != NULL);
+	
 	doc->readonly = readonly;
+	doc->changed = FALSE;
 		
 	for (n = 0; n < g_list_length (doc->views); n++)
 	{
@@ -237,18 +241,38 @@ gedit_document_text_changed_signal_connect (GeditDocument *doc)
  * Return Value: a potiner to a newly allocated string 
  **/
 gchar*
-gedit_document_get_tab_name (GeditDocument *doc)
+gedit_document_get_tab_name (GeditDocument *doc, gboolean star)
 {
 	GeditDocument *nth_doc;
 	int max_number = 0;
 	int i;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
-
+	
+	g_return_val_if_fail (doc != NULL, g_strdup ("?"));
+	
 	if (doc->filename != NULL)
 	{
-		gchar * tab_name;
-		tab_name = g_strdup_printf ("%s%s", doc->readonly?_("RO - "):"", g_basename(doc->filename));
+		gchar * tab_name = NULL;
+		gchar * unescaped_str = gnome_vfs_unescape_string_for_display (doc->filename);
+
+		if(unescaped_str == NULL) 
+		{
+			g_warning ("unescaped_str == NULL in gedit_document_get_tab_name ()");
+			unescaped_str = g_strdup (doc->filename);
+		}
+		
+		if (!doc->changed || !star)
+		{
+			tab_name = g_strdup_printf ("%s%s", doc->readonly?_("RO - "):"", g_basename(unescaped_str));
+		}
+		else
+		{			
+			tab_name = g_strdup_printf ("%s%s*", doc->readonly?_("RO - "):"", g_basename(unescaped_str));
+		}
+
+		g_free (unescaped_str);
+		
 		return tab_name;
 	}
 	else
@@ -259,14 +283,21 @@ gedit_document_get_tab_name (GeditDocument *doc)
 			{
 				nth_doc = (GeditDocument *)g_list_nth_data (mdi->children, i);
 				
-				if ( nth_doc->untitled_number > max_number)
+				if (nth_doc->untitled_number > max_number)
 				{
 					max_number = nth_doc->untitled_number;
 				}
 			}
 			doc->untitled_number = max_number + 1;
 		}
-		return _(g_strdup_printf ("%s %d", _("Untitled"), doc->untitled_number));
+		if (!doc->changed || !star)
+		{
+			return _(g_strdup_printf ("%s %d", _("Untitled"), doc->untitled_number));
+		}
+		else
+		{
+			return _(g_strdup_printf ("%s %d*", _("Untitled"), doc->untitled_number));
+		}
 	}
 }
 
@@ -398,7 +429,7 @@ gedit_document_new_with_file (const gchar *file_name)
 		return FALSE;
 	
 	gedit_window_set_widgets_sensitivity (TRUE);
-	
+
 	return TRUE;
 
 }
@@ -424,10 +455,6 @@ gedit_document_get_config_string (GnomeMDIChild *child)
 static gint
 remove_child_cb (GnomeMDI *mdi, GeditDocument *doc)
 {
-	GtkWidget *msgbox;
-	gchar *fname, *msg;
-	gint ret;
-
 	gedit_debug (DEBUG_DOCUMENT, "start");
 	
 	if (!gedit_view_active())
@@ -436,12 +463,22 @@ remove_child_cb (GnomeMDI *mdi, GeditDocument *doc)
 		return TRUE;
 	}
 
-	fname = GNOME_MDI_CHILD(doc)->name;
-
 	if (doc->changed)
 	{
+		GtkWidget *msgbox, *w;
+		gchar *fname = NULL, *msg = NULL;
+		gint ret;
+
+		w = GTK_WIDGET (g_list_nth_data(doc->views, 0));
+			
+		if(w != NULL)
+			gnome_mdi_set_active_view (mdi, w);
+
+		fname = gedit_document_get_tab_name (doc, FALSE);
+
 		msg = g_strdup_printf (_("``%s'' has been modified.  Do you wish to save it?"),
 				       fname);
+		
 		msgbox = gnome_message_box_new (msg,
 						GNOME_MESSAGE_BOX_QUESTION,
 						GNOME_STOCK_BUTTON_YES,
@@ -450,7 +487,10 @@ remove_child_cb (GnomeMDI *mdi, GeditDocument *doc)
 						NULL);
 		gnome_dialog_set_default (GNOME_DIALOG (msgbox), 2);
 		ret = gnome_dialog_run_and_close (GNOME_DIALOG (msgbox));
+
+		g_free (fname);
 		g_free (msg);
+		
 		switch (ret)
 		{
 		case 0:
@@ -604,10 +644,11 @@ gedit_mdi_init (void)
 
 
 gboolean
-gedit_document_load ( GList *file_list)
+gedit_document_load (GList *file_list)
 {
 	gchar *file_name;
-
+	gboolean can_be_created;
+	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	gedit_file_stdin (NULL);
@@ -615,20 +656,80 @@ gedit_document_load ( GList *file_list)
         /* create a file for each document in the parameter list */
 	for (;file_list; file_list = file_list->next)
 	{
-		file_name = gedit_file_convert_to_full_pathname (file_list->data);
-		if (g_file_exists (file_name))
-			gedit_document_new_with_file ((gchar *) file_name);
+		
+		const gchar* scheme; 
+		GnomeVFSURI *uri = gnome_vfs_uri_new (file_list->data);
+
+		if(uri == NULL) 
+		{
+			g_print("Wrong URI: %s\n", (gchar*)file_list->data);
+			continue;
+		}
+
+		scheme = gnome_vfs_uri_get_scheme (uri);
+
+		if ((scheme != NULL) && (strcmp (scheme, "file") == 0))
+		{
+			gchar* tmp_str;
+		        gchar* tmp_str2;
+			
+			can_be_created = TRUE;
+						
+			tmp_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);				
+			tmp_str2 = gnome_vfs_unescape_string_for_display (tmp_str);
+
+			file_name = gedit_file_convert_to_full_pathname (tmp_str);			
+			
+			g_free (tmp_str);
+			g_free (tmp_str2);
+
+			gnome_vfs_uri_unref (uri);	
+			uri = gnome_vfs_uri_new (file_name);
+		}
 		else
-			gedit_file_create_popup ((guchar *) file_name);
+		{
+			/* FIXME: hide password */
+			can_be_created = FALSE;			
+			file_name = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);		
+		}
+
+		
+		if (gnome_vfs_uri_exists (uri))
+		{
+			gedit_document_new_with_file ((gchar *) file_name);
+		}
+		else	
+		{							
+			if (can_be_created)
+			{	
+				gedit_file_create_popup ((guchar *) file_name);
+			}
+			else
+			{	
+				/* If you try to open a non existant remote file */
+				gchar *errstr = g_strdup_printf (_("An error was encountered while opening the file: \n\n%s\n\n"
+							"Please make sure the file exists."), file_name);
+	
+				gnome_app_warning (gedit_window_active_app(), errstr);
+				g_free (errstr);				
+			}
+		}
+
+		g_free (file_name);
+		gnome_vfs_uri_unref(uri);		
 	}
 
 	if (gedit_document_current() == NULL)
 		gedit_document_new ();
+	
+	g_return_val_if_fail(gedit_window_active_app() != NULL, FALSE);
+	g_return_val_if_fail(gedit_document_current() != NULL, FALSE);
 
+	gedit_window_set_widgets_sensitivity_ro (gedit_window_active_app(), gedit_document_current()->readonly);
+	
 	return FALSE;
 }
 
-
 /**
  * gedit_set_title:
  * @docname : Document name in a string, the new title
@@ -651,9 +752,13 @@ gedit_document_set_title (GeditDocument *doc)
 	if (doc->filename == NULL) {
 		docname = g_strdup_printf (_("Untitled %i"), doc->untitled_number);
 	} else {
+#if 0
 		docname = g_strdup (g_basename (doc->filename));
+#endif
+		docname = gnome_vfs_unescape_string_for_display (doc->filename);		
 	}
 
+#if 0
 	if (doc->changed) {
 		title = g_strdup_printf ("gedit: %s %s", docname, _("(modified)"));
 	} else if (doc->readonly) {
@@ -661,6 +766,15 @@ gedit_document_set_title (GeditDocument *doc)
 	} else {
 		title = g_strdup_printf ("gedit: %s", docname);
 	}
+#endif
+	if (doc->changed) {
+		title = g_strdup_printf ("gedit - [%s] %s", docname, _("(modified)"));
+	} else if (doc->readonly) {
+		title = g_strdup_printf ("gedit - [%s] %s", docname, _("(readonly)"));
+	} else {
+		title = g_strdup_printf ("gedit - [%s]", docname);
+	}
+
 
 	gtk_window_set_title (gedit_window_active(), title);
 	
@@ -788,7 +902,7 @@ gedit_document_set_undo (GeditDocument *doc, gint undo_state, gint redo_state)
 	
 	gedit_debug (DEBUG_DOCUMENT, "");
 	
-	g_return_if_fail (IS_GE_DOCUMENT(doc));
+	g_return_if_fail (GEDIT_IS_DOCUMENT(doc));
 
 	for ( n=0; n < g_list_length (doc->views); n++)
 	{
