@@ -28,6 +28,8 @@
 #include <unistd.h>  /* for gedit_stdin_open () */
 #include <stdio.h>   /* for gedit_stdin_open () */
 
+#include <libgnomevfs/gnome-vfs.h>
+
 #include "commands.h"
 #include "document.h"
 #include "view.h" 
@@ -39,8 +41,10 @@
 #include "print.h"
 #include "undo.h"
 #include "search.h"
+#include "dialogs/dialogs.h"
 
 #define GEDIT_STDIN_BUFSIZE 1024
+#define PREALLOCATED_BUF_LEN 4096
 
 GtkWidget *save_file_selector = NULL;
 #if 0
@@ -58,6 +62,7 @@ void file_close_cb(GtkWidget *widget, gpointer cbdata);
 void file_close_all_cb(GtkWidget *widget, gpointer cbdata);
 void file_revert_cb (GtkWidget *widget, gpointer cbdata);
 void file_quit_cb (GtkWidget *widget, gpointer cbdata);
+void uri_open_cb (GtkWidget *widget, gpointer cbdata);
 
        gint gedit_file_create_popup (const gchar *title);
 #if 0
@@ -76,6 +81,16 @@ static void gedit_close_all_flag_verify (guchar *function);
 gchar * gedit_file_convert_to_full_pathname (const gchar * fname);
 
 
+static void  save_all_continue (GtkWidget *widget, gpointer cbdata);
+
+typedef enum {
+	SAVE_ALL_RUNNING,
+	SAVE_ALL_ENDED
+} GeditSaveAllFlagStates;
+;
+
+static gint save_all_flag = SAVE_ALL_ENDED;
+
 typedef struct {
 	GeditDocument *doc;
 	GeditView *view;
@@ -139,98 +154,216 @@ gedit_document_insert_text_when_mapped (GeditDocument *doc, const gchar * tmp_bu
  *
  * Return value: 0 on success, 1 on error.
  */
+
 gint
 gedit_file_open (GeditDocument *doc, const gchar *fname)
 {
-	gchar *tmp_buf;
-	struct stat stats;
-	FILE *fp;
+	GnomeVFSFileInfo info;
+	GnomeVFSHandle *from_handle;
+	GnomeVFSResult  result;
+
+	GnomeVFSURI* uri;
+	const gchar* scheme;
+
+	GString *tmp_buf = NULL;
+		
 	GeditView *view;
-	
-	gint _debug;
-	
+			
 	gedit_debug (DEBUG_FILE, "");
 
 	g_return_val_if_fail (fname != NULL, 1);
-	if (doc != NULL) {
+	
+	if (doc != NULL) 
+	{
 		g_return_val_if_fail (doc->filename != fname, -1);
 	}
 
-	if (stat(fname, &stats) ||  !S_ISREG(stats.st_mode))
+	gedit_flash_va ("%s %s", _(MSGBAR_LOADING_FILE), fname);
+	
+	/* Update UI */
+	while (gtk_events_pending ())
+		  gtk_main_iteration ();
+
+	gnome_vfs_file_info_init (&info);
+	result = gnome_vfs_get_file_info (fname, 
+				 &info,
+				 (GNOME_VFS_FILE_INFO_GET_MIME_TYPE
+				 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
+
+	if ((result != GNOME_VFS_OK) || 
+	    !(info.valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE) ||
+	    (info.type != GNOME_VFS_FILE_TYPE_REGULAR))
 	{
 		gchar *errstr = g_strdup_printf (_("An error was encountered while opening the file \"%s\"."
-							      "\nPlease make sure the file exists."), fname);
+					"\nPlease make sure the file exists."), fname);
 		gnome_app_error (gedit_window_active_app(), errstr);
 		g_free (errstr);
+
+		if(result == GNOME_VFS_OK)
+			gnome_vfs_file_info_clear (&info);
+
 		return 1;
 	}
 
-#if 0
-	/* Disable this check, the users have requested that they need to open 0  bytes files */
-	if (stats.st_size  == 0)
+	if(info.valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE)
 	{
-		gchar *errstr = g_strdup_printf (_("An error was encountered while opening the file:\n\n%s\n\n"
-						    "\nPlease make sure the file is not being used by another application\n"
-						    "and that the file is not empty."), fname);
+		tmp_buf = g_string_sized_new (info.size + 1);
+	}
+	else
+	{
+		tmp_buf = g_string_sized_new (PREALLOCATED_BUF_LEN);
+	}
+
+	if (tmp_buf == NULL)
+	{
+		gchar *errstr = g_strdup_printf (_("An error was encountered while opening the file \"%s\"."
+						   "\nCould not allocate the required memory."), fname);
 		gnome_app_error (gedit_window_active_app(), errstr);
 		g_free (errstr);
-		return 1;
-	}
-#endif	
+		gnome_vfs_file_info_clear (&info);
 
-	if ((tmp_buf = g_new0 (gchar, stats.st_size + 1)) == NULL)
-	{
-		gnome_app_error (gedit_window_active_app(), _("Could not allocate the required memory."));
-		return 1;
+		return 1;	
 	}
 
-	if ((fp = fopen (fname, "r")) == NULL)
+	result = gnome_vfs_open (&from_handle, fname, GNOME_VFS_OPEN_READ);
+
+	if (result != GNOME_VFS_OK)
 	{
-		gchar *errstr = g_strdup_printf (_("gedit was unable to open the file: \n\n%s\n\n"
-						   "Make sure that you have permissions for opening the file."), fname);
+		gchar *errstr = g_strdup_printf (_("An error was encountered while reading the file: \n\n%s\n\n"
+						   "Make sure that you have permissions for opening the file. "), 
+						   fname);
 		gnome_app_error (gedit_window_active_app(), errstr);
 		g_free (errstr);
+		g_string_free(tmp_buf, TRUE);
+		gnome_vfs_file_info_clear (&info);
+
+		return 1;
+	}
+	
+	while (1) 
+	{
+		GnomeVFSFileSize bytes_read;
+		guint8           data [1025];
+		
+		result = gnome_vfs_read (from_handle, data, 1024, &bytes_read);
+		
+		if ((result != GNOME_VFS_OK) && (result != GNOME_VFS_ERROR_EOF))
+		{
+			gchar *errstr = g_strdup_printf (_("An error was encountered while reading the file: \n\n%s"), 
+						   	fname);
+			gnome_app_error (gedit_window_active_app(), errstr);
+			g_free (errstr);
+			g_string_free(tmp_buf, TRUE);
+			gnome_vfs_file_info_clear (&info);
+
+			return 1;
+		}
+
+		if (bytes_read == 0)
+		{
+			break;
+		}
+		
+		if (bytes_read >  0 && bytes_read <= 1024)
+		{
+			data [bytes_read] = '\0';
+		}
+		else 
+		{
+			gchar *errstr = g_strdup_printf (_("Internal error reading the file: \n\n%s\n\n"
+							   "Please, report this error to submit@bugs.gnome.org."), fname);
+			gnome_app_error (gedit_window_active_app(), errstr);
+			g_free (errstr);
+			g_string_free(tmp_buf, TRUE);
+			gnome_vfs_file_info_clear (&info);
+
+			return 1;
+		}
+		
+		g_string_append(tmp_buf, data);
+	}
+
+	result = gnome_vfs_close (from_handle);
+	
+	if (result != GNOME_VFS_OK)
+	{
+		gchar *errstr = g_strdup_printf (_("An error was encountered while closing the file: \n\n%s"), 
+						   fname);
+		gnome_app_error (gedit_window_active_app(), errstr);
+		g_free (errstr);
+		g_string_free(tmp_buf, TRUE);
+		gnome_vfs_file_info_clear (&info);
+
 		return 1;
 	}
 
-	_debug = fread (tmp_buf, 1, stats.st_size, fp);
-	fclose (fp);
-
-	if (doc==NULL) {
+	if (doc==NULL) 
+	{
 		doc = gedit_document_new ();
 		doc->filename = g_strdup (fname);
 		doc->untitled_number = 0;
-		gedit_document_insert_text_when_mapped (doc, tmp_buf, 0, FALSE);
-		gedit_document_set_readonly (doc, access (fname, W_OK) ? TRUE : FALSE);
-		g_free (tmp_buf);
-	} else {
+		gedit_document_insert_text_when_mapped (doc, tmp_buf->str, 0, FALSE);
+	} 
+	else 
+	{
 		if (doc->filename != NULL)
 			g_free (doc->filename);
 		doc->filename = g_strdup (fname);
 		doc->untitled_number = 0;
-		gedit_document_insert_text (doc, tmp_buf, 0, FALSE);
-		gedit_document_set_readonly (doc, access (fname, W_OK) ? TRUE : FALSE);
+		gedit_document_insert_text (doc, tmp_buf->str, 0, FALSE);
 
 		/* Set the cursor position to the start */
 		view = g_list_nth_data (doc->views, 0);
 		g_return_val_if_fail (view!=NULL, 1);
 		gedit_view_set_position (view, 0);
-		g_free (tmp_buf);
 	}
 	
+
+	/* Conservatively set doc to readonly */
+	gedit_document_set_readonly (doc, TRUE);
+
+	uri = gnome_vfs_uri_new (fname);
+
+	if (uri != NULL)
+	{
+
+		scheme = gnome_vfs_uri_get_scheme(uri);
+		
+		/* FIXME: all remote files are marked as readonly */
+		if ((scheme != NULL) && (strcmp (scheme, "file") == 0) && GNOME_VFS_FILE_INFO_LOCAL (&info))
+		{
+			gchar* tmp_str;
+
+			/*
+			tmp_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);	
+			*/
+			tmp_str = gnome_vfs_get_local_path_from_uri (fname);
+				
+			if (tmp_str != NULL)
+			{
+				gedit_document_set_readonly (doc, access (tmp_str, W_OK) ? TRUE : FALSE);
+				g_free (tmp_str);
+			}
+
+		}
+		
+		gnome_vfs_uri_unref (uri);
+	}
+	
+	g_string_free (tmp_buf, TRUE);
+	gnome_vfs_file_info_clear (&info);
+
 	doc->changed = FALSE;
 	gedit_document_set_title (doc);
 	gedit_document_text_changed_signal_connect (doc);
-
 	
 	gedit_flash_va ("%s %s", _(MSGBAR_FILE_OPENED), fname);
 	gedit_recent_add (fname);
-	gedit_recent_update_all_windows (mdi);
+	gedit_recent_update_all_windows (mdi); 
 
 	return 0;
 }
 
-
 static gint
 gedit_file_selector_key_event (GtkFileSelection *fsel, GdkEventKey *event)
 {
@@ -252,7 +385,8 @@ gedit_file_selector_key_event (GtkFileSelection *fsel, GdkEventKey *event)
 static void
 gedit_file_save_as (GeditDocument *doc)
 {
-
+	static gint ok_signal = 0;
+	
 	gedit_debug (DEBUG_FILE, "");
 
 	if (doc == NULL)
@@ -292,18 +426,60 @@ gedit_file_save_as (GeditDocument *doc)
 				    GTK_SIGNAL_FUNC(gedit_close_all_flag_clear),
 				    NULL);
 
+		/* If this save as was the result of a save all, and the user cancels the
+		   save, clear the flag */
+		gtk_signal_connect(GTK_OBJECT(save_file_selector),
+				   "delete_event",
+				   GTK_SIGNAL_FUNC(save_all_continue),
+				   NULL);
+		gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(save_file_selector)->cancel_button),
+				    "clicked",
+				    GTK_SIGNAL_FUNC(save_all_continue),
+				    NULL);
+
 		/*
 		g_print ("1. Doc->filename %s untitled #%i\n", doc->filename, doc->untitled_number);
 		*/
 		/* OK clicked */
-		gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(save_file_selector)->ok_button),
+		ok_signal = gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(save_file_selector)->ok_button),
 				    "clicked",
 				    GTK_SIGNAL_FUNC (gedit_file_save_as_ok_sel),
 				    doc);
 
 	}
 
-	gtk_window_set_title (GTK_WINDOW(save_file_selector), _("Save As..."));
+	gtk_signal_disconnect (GTK_OBJECT(GTK_FILE_SELECTION(save_file_selector)->ok_button), ok_signal);
+
+	/* OK clicked */
+	ok_signal = gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(save_file_selector)->ok_button),
+			    "clicked",
+			    GTK_SIGNAL_FUNC (gedit_file_save_as_ok_sel),
+			    doc);
+
+	if (doc->filename == NULL)
+	{
+		gchar* name = gedit_document_get_tab_name (doc, FALSE);
+		gtk_file_selection_set_filename (GTK_FILE_SELECTION (save_file_selector), name);
+		g_free (name);
+	}
+	else
+	{
+		/* FIXME: does not work well with URI */
+		gchar* name = gnome_vfs_unescape_string_for_display (doc->filename);
+		g_return_if_fail (name != NULL);
+
+		gtk_file_selection_set_filename (GTK_FILE_SELECTION (save_file_selector), name);
+		g_free (name);
+/*
+		gtk_file_selection_set_filename (GTK_FILE_SELECTION (save_file_selector), 		
+						 doc->filename);
+*/												
+	}
+
+	gtk_window_set_transient_for (GTK_WINDOW (save_file_selector), gedit_window_active ());
+
+	gtk_window_set_modal (GTK_WINDOW (save_file_selector), TRUE);
+	gtk_window_set_title (GTK_WINDOW (save_file_selector), _("Save As..."));
 
 	if (!GTK_WIDGET_VISIBLE (save_file_selector))
 	{
@@ -329,6 +505,8 @@ gedit_file_save (GeditDocument *doc, const gchar *fname)
 	FILE  *file_pointer;
 	gchar *buffer;
 	GeditView  *view;
+	gchar* unescaped_name = NULL; 
+	const gchar* old_name; 
 
 	gedit_debug (DEBUG_FILE, "");
 
@@ -338,16 +516,25 @@ gedit_file_save (GeditDocument *doc, const gchar *fname)
 	
 	g_return_val_if_fail (view!=NULL, 1);
 	
+	old_name = fname;
+	
 	if (fname == NULL)
 	{
 		if (doc->filename == NULL)
 		{
+			GtkWidget *w = GTK_WIDGET (g_list_nth_data(doc->views, 0));
+			
+			if(w != NULL)
+				gnome_mdi_set_active_view (mdi, w);
+
 			gedit_file_save_as (doc);
 			return 1;
 		}
 		else
 		{
-			fname = doc->filename;
+			unescaped_name = gnome_vfs_unescape_string_for_display (doc->filename);
+			fname = unescaped_name;
+			old_name = doc->filename;
 		}
 	}
 
@@ -380,6 +567,7 @@ gedit_file_save (GeditDocument *doc, const gchar *fname)
 		gnome_app_error (mdi->active_window, errstr);
 		g_free (errstr);
 		g_free (buffer);
+		g_free (unescaped_name);
 		gedit_close_all_flag_clear();
 		return 1;
 	}
@@ -394,6 +582,7 @@ gedit_file_save (GeditDocument *doc, const gchar *fname)
 		fclose (file_pointer);
 		g_free (errstr);
 		g_free (buffer);
+		g_free (unescaped_name);
 		gedit_close_all_flag_clear();
 		return 1;
 	}
@@ -409,17 +598,18 @@ gedit_file_save (GeditDocument *doc, const gchar *fname)
 						   "Problem to submit@bugs.gnome.org"), fname);
 		gnome_app_error (mdi->active_window, errstr);
 		g_free (errstr);
+		g_free (unescaped_name);
 		gedit_close_all_flag_clear();
 		return 1;
 	}
 
-	if (doc->filename != fname)
+	if (doc->filename != old_name)
 	{
 		if (doc->filename != NULL)
 			g_free (doc->filename);
-		doc->filename = g_strdup (fname);
+		doc->filename = g_strdup (old_name);
 	}
-	
+
 	doc->changed = FALSE;
 	doc->untitled_number = 0;
 	gedit_document_set_readonly (doc, access (fname, W_OK) ? TRUE : FALSE);
@@ -429,10 +619,50 @@ gedit_file_save (GeditDocument *doc, const gchar *fname)
 
 	gedit_flash (_(MSGBAR_FILE_SAVED));
 
+	if (unescaped_name != NULL) g_free (unescaped_name);
+
 	return 0;
 }
 
+static void 
+save_all_continue (GtkWidget *widget, gpointer cbdata)
+{
+	if (save_all_flag == SAVE_ALL_RUNNING)
+		file_save_all ();
+}
 
+void 
+file_save_all (void)
+{
+	static gint last_file_saved = -1;
+	gint i;
+	GeditDocument *doc;
+	
+	save_all_flag = SAVE_ALL_RUNNING;
+	
+	gedit_debug (DEBUG_FILE, "");		
+	
+        for (i = last_file_saved + 1; i < g_list_length (mdi->children); i++)
+	{
+		doc = (GeditDocument *)g_list_nth_data (mdi->children, i);
+		
+		if ((!doc->readonly && doc->changed))
+		{
+			gchar* fname = doc->filename;
+			
+			gedit_file_save (doc, NULL);
+			
+			if(fname == NULL)
+			{
+				last_file_saved = i;	
+				return;
+			}
+		}
+	}
+
+	last_file_saved = -1;
+	save_all_flag = SAVE_ALL_ENDED;
+}
 
 /**
  * gedit_file_stdin:
@@ -615,6 +845,13 @@ gedit_file_open_ok_sel (GtkWidget *widget, GtkWidget *file_selector_)
 	
 	gtk_widget_hide (GTK_WIDGET(file_selector));
 	
+	if (gedit_document_current())
+	{
+		g_return_if_fail(gedit_window_active_app());
+		gedit_window_set_widgets_sensitivity_ro (gedit_window_active_app(), 
+				gedit_document_current()->readonly);
+	}	
+
 	return;
 }
 
@@ -665,7 +902,10 @@ file_open_cb (GtkWidget *widget, gpointer cbdata)
 		gtk_entry_set_text (GTK_ENTRY(file_selector->selection_entry), "");
 	}
 
-	gtk_window_set_title (GTK_WINDOW(open_file_selector), _("Open File ..."));
+	gtk_window_set_transient_for (GTK_WINDOW (open_file_selector), gedit_window_active ());
+
+	gtk_window_set_modal (GTK_WINDOW (open_file_selector), TRUE);
+	gtk_window_set_title (GTK_WINDOW (open_file_selector), _("Open File ..."));
 
 	if (!GTK_WIDGET_VISIBLE (open_file_selector))
 	{
@@ -723,6 +963,11 @@ file_save_document (GeditDocument *doc)
 
 	if (doc->filename == NULL)
 	{
+		GtkWidget *w = GTK_WIDGET (g_list_nth_data(doc->views, 0));
+			
+		if(w != NULL)
+			gnome_mdi_set_active_view (mdi, w);
+
 		gedit_file_save_as (doc);
 		return FALSE;
 	}
@@ -754,19 +999,16 @@ file_save_cb (GtkWidget *widget, gpointer cbdata)
 void 
 file_save_all_cb (GtkWidget *widget, gpointer cbdata)
 {
-	int i;
-	GeditDocument *doc;
-
-	gedit_debug (DEBUG_FILE, "");
-	
-        for (i = 0; i < g_list_length (mdi->children); i++)
-	{
-		doc = (GeditDocument *)g_list_nth_data (mdi->children, i);
-		gedit_file_save (doc, NULL);
-	}
+	file_save_all();
 }
 
 
+void 
+uri_open_cb (GtkWidget *widget, gpointer cbdata)
+{
+	gedit_dialog_open_uri ();
+}
+
 static void
 gedit_file_save_as_ok_sel (GtkWidget *w, gpointer cbdata)
 {
@@ -816,7 +1058,7 @@ gedit_file_save_as_ok_sel (GtkWidget *w, gpointer cbdata)
 		}
 	}
 	    
-	if (gedit_file_save(doc, file_name) != 0)
+	if (gedit_file_save (doc, file_name) != 0)
 	{
 		gedit_flash (_("Error saving file!"));
 		g_free (file_name);
@@ -852,11 +1094,20 @@ gedit_file_save_as_ok_sel (GtkWidget *w, gpointer cbdata)
 	default:
 		g_return_if_fail (FALSE);
 	}
+
+	if(save_all_flag == SAVE_ALL_RUNNING)
+	{
+		file_save_all();
+	}
 	
 	/* We need to set the revert menu item sensitivity if we where saving
 	   as from an Untitled doc */
-	gedit_window_set_view_menu_sensitivity (gedit_window_active_app());
+	gedit_window_set_view_menu_sensitivity (gedit_window_active_app ());
 
+	if(gedit_window_active_app () != NULL)
+	{
+		gedit_window_set_widgets_sensitivity_ro (gedit_window_active_app (), FALSE);	
+	}
 }
 
 
