@@ -47,7 +47,7 @@ GnomeUIInfo popup_menu[] =
 
 	GNOMEUIINFO_MENU_SAVE_ITEM  (file_save_cb, NULL),
 	GNOMEUIINFO_MENU_CLOSE_ITEM (file_close_cb, NULL),
-	GNOMEUIINFO_MENU_PRINT_ITEM (file_print_cb, NULL),
+	GNOMEUIINFO_MENU_PRINT_ITEM (gedit_print_cb, NULL),
 
 	GNOMEUIINFO_SEPARATOR,
 
@@ -75,20 +75,33 @@ GnomeUIInfo gedit_file_menu[] =
 				NULL,
 				file_revert_cb, 
 				GNOME_STOCK_MENU_REFRESH),
+
 	GNOMEUIINFO_SEPARATOR,
 	/* Recent files ... */
 	GNOMEUIINFO_SEPARATOR,
+	
+	GNOMEUIINFO_ITEM_STOCK (N_("Open from _URI..."),
+			  N_("Open a file from a specified URI"),
+			  uri_open_cb,
+			  GNOME_STOCK_MENU_OPEN),
+/*	
+	GNOMEUIINFO_ITEM_STOCK (N_("Save _to URI..."),
+			  N_("Save the current file to a specified URI"),
+			  NULL,
+			  GNOME_STOCK_MENU_SAVE_AS),
+*/	
+	GNOMEUIINFO_SEPARATOR,
 
-	GNOMEUIINFO_MENU_PRINT_ITEM (file_print_cb, NULL),
+	GNOMEUIINFO_MENU_PRINT_ITEM (gedit_print_cb, NULL),
 #if 0	
 	GNOMEUIINFO_ITEM_DATA (
 		N_("Print pre_view"), N_("Print preview"),
-		file_print_preview_cb, NULL, NULL),
+		gedit_print_preview_cb, NULL, NULL),
 /*		file_print_preview_cb, NULL, preview_xpm),*/
 #else	
 	GNOMEUIINFO_ITEM (N_("Print preview..."),
 			  N_("Preview data to be printed"),
-			  file_print_preview_cb, NULL),
+			  gedit_print_preview_cb, NULL),
 #endif	
 
 	GNOMEUIINFO_SEPARATOR, 
@@ -98,6 +111,9 @@ GnomeUIInfo gedit_file_menu[] =
 				N_("Close All Open Files"),
 				file_close_all_cb,
 				GNOME_STOCK_MENU_CLOSE),
+
+	GNOMEUIINFO_SEPARATOR, 
+
 	GNOMEUIINFO_MENU_EXIT_ITEM (file_quit_cb, NULL),
 
 	GNOMEUIINFO_END
@@ -237,7 +253,7 @@ GnomeUIInfo toolbar_data[] =
 		NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_CLOSE
 	},
 	{
-		GNOME_APP_UI_ITEM, N_("Print"), N_("Print the current file"), file_print_cb,
+		GNOME_APP_UI_ITEM, N_("Print"), N_("Print the current file"), gedit_print_cb,
 		NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PRINT
 	},
 	GNOMEUIINFO_SEPARATOR,
@@ -262,6 +278,7 @@ GnomeUIInfo toolbar_data[] =
 		GNOME_APP_UI_ITEM, N_("Paste"), N_("Paste the clipboard"), edit_paste_cb,
 		NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PASTE
 	},
+	GNOMEUIINFO_SEPARATOR,
 	{
 		GNOME_APP_UI_ITEM, N_("Find"), N_("Search for a string"), gedit_find_cb,
 		NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_SEARCH
