@@ -23,8 +23,8 @@
  *   Chema Celorio <chema@celorio.com>
  */
 
-#ifndef __UTILS_H__
-#define __UTILS_H__
+#ifndef __GEDIT_UTILS_H__
+#define __GEDIT_UTILS_H__
 
 typedef enum {
 	GEDIT_DEBUG_VIEW,
@@ -92,7 +92,7 @@ gint	gedit_utils_is_program (gchar * program, gchar* default_name);
 
 void	gedit_debug (gint section, gchar *file, gint line, gchar* function, gchar* format, ...);
 void 	gedit_utils_delete_temp (gchar* file_name);
-gchar*	gedit_utils_create_temp_from_doc (GeditDocument *doc, gint number);
+gchar *	gedit_utils_create_temp_from_doc (GeditDocument *doc, gint number);
 void	gedit_utils_error_dialog (gchar *error_message, GtkWidget *widget);
 
-#endif /* __UTILS_H__ */
+#endif /* __GEDIT_UTILS_H__ */
