@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /* Clean part of this 10-00, by Chema. divided the ugly code and the nice one */
 /*
  * gedit
@@ -359,7 +359,7 @@ gedit_view_get_position (GeditView *view)
 void
 gedit_view_set_readonly (GeditView *view, gint readonly)
 {
-	gchar * doc_name;
+	gchar * doc_name = NULL;
 
 	gedit_debug (DEBUG_VIEW, "");
 
@@ -368,12 +368,28 @@ gedit_view_set_readonly (GeditView *view, gint readonly)
 	view->readonly = readonly;
 	gtk_text_set_editable (view->text, !view->readonly);
 	
-	doc_name = gedit_document_get_tab_name (view->doc);
+	doc_name = gedit_document_get_tab_name (view->doc, TRUE);
 	gnome_mdi_child_set_name (GNOME_MDI_CHILD (view->doc), doc_name);
 	gedit_document_set_title (view->doc);
 	g_free (doc_name);
 }
 
+/**
+ * gedit_view_get_document:
+ * @view: 
+ * 
+ * get the document from a view
+ * 
+ * Return Value: document for the @view, NULL on error
+ **/
+GeditDocument *
+gedit_view_get_document (GeditView *view)
+{
+        if (view == NULL)
+                return NULL;
+
+        return view->doc;
+}
 
 
 
@@ -488,7 +504,7 @@ gedit_event_key_press (GtkWidget *w, GdkEventKey *event)
 			file_save_cb (w, NULL);
 	    		break;
 		case 'p':
-	    		file_print_cb (w, NULL, FALSE);
+	    		gedit_print_cb (w, NULL);
 	    		break;
 		case 'n':
 			return FALSE;
@@ -707,11 +723,11 @@ static void
 gedit_view_init (GeditView *view)
 {
 	GtkWidget *window;
-	GtkWidget *menu;
+	static GtkWidget *menu = NULL;
 	GtkStyle *style;
 	GdkColor *bg;
 	GdkColor *fg;
-
+	
 	gedit_debug (DEBUG_VIEW, "");
 
 	/* Vbox */
@@ -729,6 +745,7 @@ gedit_view_init (GeditView *view)
 	
 	/* Create and configure the GtkText Widget */
 	view->text = GTK_TEXT (gtk_text_new (NULL, NULL));
+	gedit_view_set_tab_size (view, settings->tab_size);
 	gtk_text_set_editable  (view->text, !view->readonly);
 	gtk_text_set_word_wrap (view->text, settings->word_wrap);
 	
@@ -777,12 +794,19 @@ gedit_view_init (GeditView *view)
 	gtk_style_unref (style);
 
 	/* Popup Menu */
-	menu = gnome_popup_menu_new (popup_menu);
+	if (menu == NULL) 
+	{
+		menu = gnome_popup_menu_new (popup_menu);
+		/* popup menu will not be destroyed when all the view are closed
+		 * FIXME: destroy popup before exiting the program*/
+		gtk_widget_ref(menu);
+	}
+
+	/* The same popup menu is attached to all views */
 	gnome_popup_menu_attach (menu, GTK_WIDGET (view->text), view);
 
 	gtk_widget_show_all   (GTK_WIDGET (view));
 	gtk_widget_grab_focus (GTK_WIDGET (view->text));
-
 }
 
 
@@ -817,6 +841,7 @@ gedit_view_changed_cb (GnomeMDI *mdi, GtkWidget *old_view)
 	gnome_app_install_menu_hints(view->app, gnome_mdi_get_child_menu_info(view->app));
 
 	gedit_window_set_view_menu_sensitivity (gedit_window_active_app());
+	gedit_window_set_widgets_sensitivity_ro (gedit_window_active_app(), doc->readonly);
 
 	gedit_debug (DEBUG_VIEW, "end");
 }
@@ -825,6 +850,7 @@ void
 gedit_view_text_changed_cb (GtkWidget *w, gpointer cbdata)
 {
 	GeditView *view;
+	gchar* doc_name;
 	
 	gedit_debug (DEBUG_VIEW, "");
 
@@ -841,6 +867,10 @@ gedit_view_text_changed_cb (GtkWidget *w, gpointer cbdata)
 
 	/* Set the title ( so that we add the "modified" string to it )*/
 	gedit_document_set_title (view->doc);
+	
+	doc_name = gedit_document_get_tab_name (view->doc, TRUE);
+	gnome_mdi_child_set_name (GNOME_MDI_CHILD (view->doc), doc_name);
+	g_free(doc_name);
 }
 
 
@@ -945,6 +975,7 @@ gedit_view_add_cb (GtkWidget *widget, gpointer data)
 		gedit_view_set_window_position (view, 0);
 		g_free (buffer);
 		gedit_window_set_view_menu_sensitivity (gedit_window_active_app());
+		gedit_window_set_widgets_sensitivity_ro (gedit_window_active_app(), view->doc->readonly);
 	}
 
 }
@@ -973,6 +1004,7 @@ gedit_view_remove (GeditView *view)
 	gedit_document_set_title (doc);
 
 	gedit_window_set_view_menu_sensitivity (gedit_window_active_app());
+	gedit_window_set_widgets_sensitivity_ro (gedit_window_active_app(), doc->readonly);
 }
 
 void
@@ -1195,3 +1227,31 @@ gedit_view_set_undo (GeditView *view, gint undo_state, gint redo_state)
 	}
 }
 		
+
+/**
+ * gedit_view_set_tab_size:
+ * @view: 
+ * @tab_size: 
+ * 
+ * Set the gtk_text tab size
+ **/
+void
+gedit_view_set_tab_size (GeditView *view, gint tab_size)
+{
+	GList *l;
+	gint i;
+
+	g_return_if_fail (GEDIT_IS_VIEW (view));
+
+	l = GTK_TEXT (view->text)->tab_stops;
+	g_list_free (l);
+		
+	l = NULL;
+	for (i = 0; i < 50; i++)
+		l = g_list_prepend (l, GINT_TO_POINTER (tab_size));
+	GTK_TEXT (view->text)->tab_stops = l;
+
+}
+
+
+
